@@ -5,17 +5,8 @@ pub(crate) mod deque;
 pub(crate) mod error;
 pub(crate) mod frequency_sketch;
 pub(crate) mod thread_pool;
-pub(crate) mod unsafe_weak_pointer;
-
-// targe_has_atomic is more convenient but yet unstable (Rust 1.55)
-// https://github.com/rust-lang/rust/issues/32976
-// #[cfg_attr(target_has_atomic = "64", path = "common/time_atomic64.rs")]
-
-#[cfg_attr(feature = "atomic64", path = "common/atomic_time.rs")]
-#[cfg_attr(not(feature = "atomic64"), path = "common/atomic_time_compat.rs")]
-pub(crate) mod atomic_time;
-
 pub(crate) mod time;
+pub(crate) mod unsafe_weak_pointer;
 
 // Note: `CacheRegion` cannot have more than four enum variants. This is because
 // `crate::{sync,unsync}::DeqNodes` uses a `tagptr::TagNonNull<DeqNode<T>, 2>`
