@@ -2,10 +2,9 @@ use super::{iter::DashMapIter, Iter};
 use crate::{
     common::{
         self,
-        atomic_time::AtomicInstant,
         deque::{DeqNode, Deque},
         frequency_sketch::FrequencySketch,
-        time::{CheckedTimeOps, Clock, Instant},
+        time::{atomic_time::AtomicInstant, CheckedTimeOps, Clock, Instant},
         CacheRegion,
     },
     sync::{
